@@ -12,11 +12,7 @@ class DataSource(metaclass=abc.ABCMeta):
         return "easytrade_backtestorder"
 
     @abc.abstractmethod
-    def connect(self):
-        pass
-
-    @abc.abstractmethod
-    def close(self):
+    def getConnection(self):
         pass
 
     @abc.abstractmethod
@@ -66,3 +62,7 @@ class DataSource(metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def truncateBackTestOrders(self):
         pass
+
+    @abc.abstractmethod
+    def deleteBackTestOrders(self, backTestCode):
+        pass
