@@ -1,4 +1,6 @@
 import pymysql
+from dbutils.pooled_db import PooledDB
+from pymysql.cursors import DictCursor
 
 from easytradetesting.BackTestOrder import BackTestOrder
 from easytradetesting.DataSource import DataSource
@@ -9,33 +11,33 @@ from easytradesdk.entity.Kline import Kline
 class MySqlDataSource(DataSource):
 
     def __init__(self, host="localhost", port=3306, user=None, password=None, database=None):
+
         self.__host = host
         self.__port = port
         self.__user = user
         self.__password = password
         self.__database = database
-        self.__connection = self.connect()
 
-    def connect(self):
+        self.__pool = PooledDB(
+            # 使用链接数据库的模块
+            creator=pymysql,
+            host=self.__host,
+            port=self.__port,
+            user=self.__user,
+            password=self.__password,
+            database=self.__database,
+            charset="utf8",
+            cursorclass=DictCursor,
+            maxconnections=50,
+            mincached=5,
+            maxcached=10,
+            blocking=True,
+            maxusage=None,
+            ping=1
+        )
 
-        """
-            连接数据库
-            :return:
-        """
-        return pymysql.connect(host=self.__host,
-                               port=self.__port,
-                               user=self.__user,
-                               password=self.__password,
-                               database=self.__database,
-                               charset="utf8")
-
-    def close(self):
-
-        """
-            关闭数据库连接
-            :return:
-        """
-        self.__connection.close()
+    def getConnection(self):
+        return self.__pool.connection()
 
     def createKlineTable(self, tc, symbol, klinePeriod):
 
@@ -72,12 +74,14 @@ class MySqlDataSource(DataSource):
         """
 
         sql = sql.replace("${tableName}", DataSource.resolveKlineTableName(tc, symbol, klinePeriod))
-        cursor = self.__connection.cursor()
+        conn = self.getConnection()
+        cursor = conn.cursor()
 
         try:
             cursor.execute(sql)
         finally:
             cursor.close()
+            conn.close()
 
     def saveKline(self, tc, symbol, klinePeriod, kline):
 
@@ -89,15 +93,19 @@ class MySqlDataSource(DataSource):
             :param klinePeriod:   k线时间戳
             :return:
         """
-        cursor = self.__connection.cursor()
+
         if kline is None:
             return
 
-        sql = 'insert ignore into ' + DataSource.resolveKlineTableName(tc, symbol, klinePeriod) + \
+        sql = 'insert into ' + DataSource.resolveKlineTableName(tc, symbol, klinePeriod) + \
               ' (timeMills,endTimeMills,startTime,endTime,tc,symbol,timeInterval,openingPrice,closingPrice,highPrice,lowPrice,' \
               ' volume,quantity,cnt,buyVolume,buyQuantity)' \
-              ' values(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)'
+              ' values(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s) ' \
+              ' ON DUPLICATE KEY UPDATE openingPrice=%s,closingPrice=%s,highPrice=%s,lowPrice=%s,volume=%s,quantity=%s,cnt=%s,buyVolume=%s,' \
+              ' buyQuantity=%s'
 
+        conn = self.getConnection()
+        cursor = conn.cursor()
         try:
             cursor.execute(sql, (kline.timeMills, kline.endTimeMills,
                                  kline.startTime, kline.endTime,
@@ -105,10 +113,16 @@ class MySqlDataSource(DataSource):
                                  kline.openingPrice, kline.closingPrice,
                                  kline.highPrice, kline.lowPrice,
                                  kline.volume, kline.quantity,
-                                 kline.cnt, kline.buyVolume, kline.buyQuantity))
-            self.__connection.commit()
+                                 kline.cnt, kline.buyVolume, kline.buyQuantity,
+                                 kline.openingPrice, kline.closingPrice,
+                                 kline.highPrice, kline.lowPrice,
+                                 kline.volume, kline.quantity,
+                                 kline.cnt, kline.buyVolume, kline.buyQuantity
+                                 ))
+            conn.commit()
         finally:
             cursor.close()
+            conn.close()
 
     def saveKlines(self, tc, symbol, klinePeriod, klines):
 
@@ -120,8 +134,10 @@ class MySqlDataSource(DataSource):
             :param klines:  K线对象列表
             :return:
         """
-        cursor = self.__connection.cursor()
+
         if klines:
+            conn = self.getConnection()
+            cursor = conn.cursor()
             data = ((klines[i].timeMills, klines[i].endTimeMills,
                      klines[i].startTime, klines[i].endTime,
                      klines[i].tc, klines[i].symbol, klines[i].timeInterval,
@@ -135,11 +151,12 @@ class MySqlDataSource(DataSource):
                   ' values(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)'
             try:
                 cursor.executemany(sql, data)
-                self.__connection.commit()
+                conn.commit()
             except Exception as e:
-                self.__connection.rollback()
+                conn.rollback()
             finally:
                 cursor.close()
+                conn.close()
 
     def queryKline(self, tc, symbol, klinePeriod, startTimeMills=None, endTimeMills=None, limit=200):
 
@@ -167,13 +184,15 @@ class MySqlDataSource(DataSource):
         sql += " order by timeMills asc limit %s"
         params.append(limit)
 
-        cursor = self.__connection.cursor(cursor=pymysql.cursors.DictCursor)
+        conn = self.getConnection()
+        cursor = conn.cursor()
 
         try:
             cursor.execute(sql, params)
             return Serializer.dictListToObjectList(cursor.fetchall(), Kline)
         finally:
             cursor.close()
+            conn.close()
 
     def queryLatestKline(self, tc, symbol, klinePeriod, endTimeMills=None, limit=200):
 
@@ -194,7 +213,8 @@ class MySqlDataSource(DataSource):
         sql += " order by timeMills desc limit %s"
         params.append(limit)
 
-        cursor = self.__connection.cursor(cursor=pymysql.cursors.DictCursor)
+        conn = self.getConnection()
+        cursor = conn.cursor()
 
         try:
             cursor.execute(sql, params)
@@ -204,6 +224,7 @@ class MySqlDataSource(DataSource):
             return klines
         finally:
             cursor.close()
+            conn.close()
 
     def countKline(self, tc, symbol, klinePeriod, startTimeMills=None, endTimeMills=None):
         """
@@ -225,19 +246,22 @@ class MySqlDataSource(DataSource):
             sql += " and timeMills <= %s"
             params.append(endTimeMills)
 
-        cursor = self.__connection.cursor(cursor=pymysql.cursors.DictCursor)
+        conn = self.getConnection()
+        cursor = conn.cursor()
 
         try:
             cursor.execute(sql, params)
             return cursor.fetchone()['cnt']
         finally:
             cursor.close()
+            conn.close()
 
     def createBackTestOrderTable(self):
 
         sql = """
                 CREATE TABLE IF NOT EXISTS `${tableName}` (
                 `id` int(10) NOT NULL AUTO_INCREMENT,
+                `backTestCode` varchar(200) NOT NULL COMMENT '回测编号',
                 `clientOrderId` varchar(50) NOT NULL COMMENT '订单id',
                 `tc` varchar(30) DEFAULT NULL COMMENT '交易所',
                 `symbol` varchar(30) DEFAULT NULL COMMENT '交易标的',
@@ -256,20 +280,26 @@ class MySqlDataSource(DataSource):
                 `posBeforeDeal` json DEFAULT NULL COMMENT '交易前仓位快照',
                 `posAfterDeal` json DEFAULT NULL COMMENT '交易后仓位快照',
                 `posTotalAmountAfterDeal` decimal(30,10) DEFAULT NULL COMMENT '交易后仓位总价值金额',
+                `timeCreated` datetime(3) DEFAULT CURRENT_TIMESTAMP(3),
+                `timeModified` datetime(3) DEFAULT CURRENT_TIMESTAMP(3) ON UPDATE CURRENT_TIMESTAMP(3),
                 PRIMARY KEY (`id`),
                 UNIQUE KEY `uk_clientOrderId` (`clientOrderId`) USING BTREE,
+                KEY `idx_backTestCode` (`backTestCode`) USING BTREE,
                 KEY `idx_time` (`time`) USING BTREE,
                 KEY `idx_timeMills` (`timeMills`) USING BTREE
                 ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;
             """
 
         sql = sql.replace("${tableName}", DataSource.resolveBackTestOrderTableName())
-        cursor = self.__connection.cursor()
+
+        conn = self.getConnection()
+        cursor = conn.cursor()
 
         try:
             cursor.execute(sql)
         finally:
             cursor.close()
+            conn.close()
 
     def saveBackTestOrder(self, backTestOrder):
 
@@ -278,31 +308,36 @@ class MySqlDataSource(DataSource):
             :param backTestOrder:
             :return:
         """
-        cursor = self.__connection.cursor()
 
-        data = (backTestOrder.clientOrderId, backTestOrder.tc,
-                backTestOrder.symbol, backTestOrder.price,
-                backTestOrder.totalQty, backTestOrder.dealQty,
-                backTestOrder.type, backTestOrder.side,
-                backTestOrder.status, backTestOrder.actualDealQty,
-                backTestOrder.actualDealAmount, backTestOrder.time, backTestOrder.timeMills,
-                Serializer.objectToJson(backTestOrder.orderSignal), Serializer.objectToJson(backTestOrder.orderData),
-                Serializer.objectToJson(backTestOrder.posBeforeDeal), Serializer.objectToJson(backTestOrder.posAfterDeal),
-                backTestOrder.posTotalAmountAfterDeal)
+        conn = self.getConnection()
+        cursor = conn.cursor()
+
+        data = (
+            backTestOrder.backTestCode,
+            backTestOrder.clientOrderId, backTestOrder.tc,
+            backTestOrder.symbol, backTestOrder.price,
+            backTestOrder.totalQty, backTestOrder.dealQty,
+            backTestOrder.type, backTestOrder.side,
+            backTestOrder.status, backTestOrder.actualDealQty,
+            backTestOrder.actualDealAmount, backTestOrder.time, backTestOrder.timeMills,
+            Serializer.objectToJson(backTestOrder.orderSignal), Serializer.objectToJson(backTestOrder.orderData),
+            Serializer.objectToJson(backTestOrder.posBeforeDeal), Serializer.objectToJson(backTestOrder.posAfterDeal),
+            backTestOrder.posTotalAmountAfterDeal)
 
         sql = 'insert into ' + DataSource.resolveBackTestOrderTableName() + \
-              ' (clientOrderId,tc,symbol,price,totalQty,dealQty,type,side,status,actualDealQty,' \
+              ' (backTestCode,clientOrderId,tc,symbol,price,totalQty,dealQty,type,side,status,actualDealQty,' \
               ' actualDealAmount,time,timeMills,orderSignal,orderData,posBeforeDeal,posAfterDeal,posTotalAmountAfterDeal)' \
-              ' values(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)'
+              ' values(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)'
         try:
             cursor.execute(sql, data)
-            backTestOrder.id = self.__connection.insert_id()
-            self.__connection.commit()
+            backTestOrder.id = cursor.lastrowid
+            conn.commit()
         except Exception as e:
-            self.__connection.rollback()
+            conn.rollback()
             raise Exception(e)
         finally:
             cursor.close()
+            conn.close()
 
     def saveBackTestOrders(self, backTestOrders, deleteDataBeforeSave=False):
 
@@ -313,30 +348,34 @@ class MySqlDataSource(DataSource):
             :return:
         """
 
-        cursor = self.__connection.cursor()
+        conn = self.getConnection()
+        cursor = conn.cursor()
 
-        data = ((backTestOrders[i].clientOrderId, backTestOrders[i].tc,
-                 backTestOrders[i].symbol, backTestOrders[i].price,
-                 backTestOrders[i].totalQty, backTestOrders[i].dealQty,
-                 backTestOrders[i].type, backTestOrders[i].side,
-                 backTestOrders[i].status, backTestOrders[i].actualDealQty,
-                 backTestOrders[i].actualDealAmount, backTestOrders[i].time, backTestOrders[i].timeMills,
-                 Serializer.objectToJson(backTestOrders[i].orderSignal), Serializer.objectToJson(backTestOrders[i].orderData),
-                 Serializer.objectToJson(backTestOrders[i].posBeforeDeal), Serializer.objectToJson(backTestOrders[i].posAfterDeal),
-                 backTestOrders[i].posTotalAmountAfterDeal) for i in range(0, len(backTestOrders)))
+        data = ((
+            backTestOrders[i].backTestCode,
+            backTestOrders[i].clientOrderId, backTestOrders[i].tc,
+            backTestOrders[i].symbol, backTestOrders[i].price,
+            backTestOrders[i].totalQty, backTestOrders[i].dealQty,
+            backTestOrders[i].type, backTestOrders[i].side,
+            backTestOrders[i].status, backTestOrders[i].actualDealQty,
+            backTestOrders[i].actualDealAmount, backTestOrders[i].time, backTestOrders[i].timeMills,
+            Serializer.objectToJson(backTestOrders[i].orderSignal), Serializer.objectToJson(backTestOrders[i].orderData),
+            Serializer.objectToJson(backTestOrders[i].posBeforeDeal), Serializer.objectToJson(backTestOrders[i].posAfterDeal),
+            backTestOrders[i].posTotalAmountAfterDeal) for i in range(0, len(backTestOrders)))
 
         sql = 'insert into ' + DataSource.resolveBackTestOrderTableName() + \
-              ' (clientOrderId,tc,symbol,price,totalQty,dealQty,type,side,status,actualDealQty,' \
+              ' (backTestCode,clientOrderId,tc,symbol,price,totalQty,dealQty,type,side,status,actualDealQty,' \
               ' actualDealAmount,time,timeMills,orderSignal,orderData,posBeforeDeal,posAfterDeal,posTotalAmountAfterDeal)' \
-              ' values(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)'
+              ' values(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)'
         try:
             cursor.executemany(sql, data)
-            self.__connection.commit()
+            conn.commit()
         except Exception as e:
-            self.__connection.rollback()
+            conn.rollback()
             raise Exception(e)
         finally:
             cursor.close()
+            conn.close()
 
     def getBackTestOrder(self, clientOrderId):
 
@@ -351,13 +390,14 @@ class MySqlDataSource(DataSource):
 
         sql = "select * from " + DataSource.resolveBackTestOrderTableName() + " where clientOrderId = %s"
 
-        cursor = self.__connection.cursor(cursor=pymysql.cursors.DictCursor)
-
+        conn = self.getConnection()
+        cursor = conn.cursor()
         try:
             cursor.execute(sql, [clientOrderId])
             return Serializer.dictToObject(cursor.fetchone(), BackTestOrder)
         finally:
             cursor.close()
+            conn.close()
 
     def queryBackTestOrders(self, startTimeMills=None, endTimeMills=None, tc=None, symbol=None, limit=50):
 
@@ -375,10 +415,10 @@ class MySqlDataSource(DataSource):
 
         if tc:
             params.append(tc)
-            sql += "and tc= %s"
+            sql += " and tc= %s"
         if symbol:
             params.append(symbol)
-            sql += "and symbol= %s"
+            sql += " and symbol= %s"
         if startTimeMills:
             params.append(startTimeMills)
             sql += " and timeMills >= %s"
@@ -387,20 +427,43 @@ class MySqlDataSource(DataSource):
             sql += " and timeMills <= %s"
         params.append(limit)
         sql += " order by timeMills asc limit %s"
-        cursor = self.__connection.cursor(cursor=pymysql.cursors.DictCursor)
+
+        conn = self.getConnection()
+        cursor = conn.cursor()
 
         try:
             cursor.execute(sql, params)
             return Serializer.dictListToObjectList(cursor.fetchall(), BackTestOrder)
         finally:
             cursor.close()
+            conn.close()
 
     def truncateBackTestOrders(self):
 
         sql = "truncate table " + DataSource.resolveBackTestOrderTableName()
-        cursor = self.__connection.cursor()
+
+        conn = self.getConnection()
+        cursor = conn.cursor()
 
         try:
             cursor.execute(sql)
         finally:
             cursor.close()
+            conn.close()
+
+    def deleteBackTestOrders(self, backTestCode):
+
+        sql = "delete from " + DataSource.resolveBackTestOrderTableName() + " where backTestCode= %s"
+
+        conn = self.getConnection()
+        cursor = conn.cursor()
+
+        try:
+            cursor.execute(sql, [backTestCode])
+            conn.commit()
+        except Exception as e:
+            conn.rollback()
+            raise Exception(e)
+        finally:
+            cursor.close()
+            conn.close()
